"""
Extras import
"""

from xraptor.antenna_implementations.memory import MemoryAntenna
from xraptor.core.interfaces import Antenna as IAntenna

__all__ = ["IAntenna", "MemoryAntenna"]

# Redis edition extra
try:
    import redis.asyncio as redis
    from .redis import RedisAntenna

    __all__ += "RedisAntenna"
except ImportError as error:  # pragma: no cover
    pass



import logging
from typing import AsyncIterator

import redis.asyncio as redis
from decouple import config

from xraptor.core.interfaces import Antenna


class RedisAntenna(Antenna):
    def __init__(self):
        try:
            self._redis = redis.Redis.from_url(url=config("X_RAPTOR_REDIS_URL"))
        except Exception as error:  # pylint: disable=W0718
            logging.error(error)

    async def subscribe(self, antenna_id: str) -> AsyncIterator[str]:
        pubsub = self._redis.pubsub()
        await pubsub.subscribe(antenna_id)
        async for message in pubsub.listen():
            if message["type"] == "message":
                yield message["data"]

    async def post(self, antenna_id: str, message: str):
        await self._redis.publish(antenna_id, message)

    async def is_alive(self, antenna_id: str) -> bool:
        num_subscribers = await self._redis.execute_command(
            "PUBSUB", "NUMSUB", antenna_id
        )
        return bool(num_subscribers[1])



import json
from dataclasses import dataclass

from xraptor.domain.methods import MethodType


@dataclass(slots=True, frozen=True)
class Request:
    request_id: str
    payload: str
    header: dict
    route: str
    method: MethodType

    def __post_init__(self):
        assert isinstance(self.request_id, str), f"request_id is not of type {str}"
        assert isinstance(self.payload, str), f"header is not of type {str}"
        assert isinstance(self.header, dict), f"payload is not of type {dict}"
        assert isinstance(self.route, str), f"payload is not of type {str}"
        assert isinstance(
            self.method, MethodType
        ), f"payload is not of type {MethodType}"

    def json(self) -> str:
        """
        return a string data representation
        :return:
        """
        return json.dumps(
            {
                "request_id": self.request_id,
                "payload": self.payload,
                "header": self.header,
                "route": self.route,
                "method": self.method.value,
            }
        )

    @classmethod
    def from_message(cls, message: str | bytes):
        """
        cast string message to a valid Request object instance
        :param message: json like string
        :return: Request instance
        """

        message_data = json.loads(message)

        return cls(
            request_id=message_data["request_id"],
            payload=message_data["payload"],
            header=message_data["header"],
            route=message_data["route"],
            method=MethodType[message_data["method"]],
        )



import json
from dataclasses import dataclass


@dataclass(slots=True, frozen=True)
class Response:
    request_id: str
    payload: str
    header: dict

    def __post_init__(self):
        assert isinstance(self.request_id, str), f"request_id is not of type {str}"
        assert isinstance(self.header, dict), f"header is not of type {dict}"
        assert isinstance(self.payload, str), f"payload is not of type {str}"

    @classmethod
    def create(cls, request_id: str, header: dict, payload: str):
        """
        create a new valid Response object instance
        :param request_id: the origin request id
        :param header: string key value map
        :param payload: string payload, normally json data
        :return: Response instance
        """
        return cls(
            request_id=request_id,
            payload=payload,
            header=header,
        )

    def json(self):
        """
        return a string data representation
        :return:
        """
        return json.dumps(
            {
                "request_id": self.request_id,
                "payload": self.payload,
                "header": self.header,
            }
        )
